# -*- coding: utf-8 -*-
"""
Created on Thu Dec 10 16:09:34 2015
@author: Peter Meisrimel
"""
from __future__ import division
import numpy as np

## Checks if string is float
# @param string input string to check
# @return (boolean)
def is_number(string):
    try:
        float(string)
        return True
    except ValueError:
        return False

## Checks if string is list
# @param string input string to check
# @return (boolean)
def is_list(string):
    if (string[0] != '[') or (string[-1] != ']'):
        return False
    else:
        string = string[1:-1]
        aux = string.split(',')
        for i in aux:
            if not is_number(i):
                return False
    return True
        
## Converts string list to vector \n
# does not double check if input is convertable
# @param string input string to convert
# @return (np.array) converted string
def convert_string_to_vector(string):
    string = string[1:-1] 
    aux = string.split(',')
    return np.array([float(i) for i in aux])
    
## Checks if input string is number/list and possibly converts accordingly
# @param string input string to convert
# @return (string/int/float/np.array) converted string
def convert(string):
    if is_number(string):
        if '.' in string or 'e' in string:
            return float(string)
        else:
            return int(string)
    else: # is no number, String or List left as options
        if is_list(string): # actual list
            return convert_string_to_vector(string)
        else: # actual strings stay unchanged
            return string

## Reads input file and saves content to dictionary
# @param filename file to read
# @return (dictionary) formated file
def read_parameters_from_file(filename):
    dic = {}
    with open(filename,'r') as myfile:
        for line in myfile:
            reading = line.split()
            if len(reading) == 0: # empty line, skip
                continue
            if reading[0][0] == '#': # commented line, skip
                continue
            if len(reading) > 3: # too many entries, gather remaining ones in reading[2]
                for i in reading[3:]:
                    reading[2] += ' ' + i
                reading = reading[:3]
            if reading[1] != '=': # no equal sign, Error
                raise Exception('Invalid file format, no = found in line')
            del reading[1] # remove equal sign
            reading[0] = reading[0].lower() # convert left side to lower case
            dic[reading[0]] = convert(reading[1])
    if not myfile.closed:
        print('Warning, file not closed after reading')
    return dic