# -*- coding: utf-8 -*-
"""
Created on Fri Jan  5 14:48:34 2018
@author: Peter Meisrimel
"""
from __future__ import division
import scipy as sp
import pylab as pl
import numpy as np
pl.close('all')

t_end = 2
k = -1
A = np.array([[-1., 1.], [0., k]])

def f(t, x):
    return A@x

u0 = np.array([1., 1.])

def scheme(t, dt, u):
    k1 = f(t, u)
    k2 = f(t + dt/2, u + dt/2*k1)
    k3 = f(t + dt/2, u + dt/2*k2)
    k4 = f(t + dt, u + dt*k3)
    return (u + dt/6*(k1 + 2*k2 + 2*k3 + k4), 
            u + dt/3*(k1 + k2 + k4),
            u + dt/24*(5*k1 + 4*k2 + 4*k3-k4)) # third order for midpoint

def ex_sol(t, u):
    return sp.linalg.expm(t*A)@u

## functional j and exact solution
def get_func(func):
    if k != -1:
        sol_end = np.array([np.exp(-t_end)*((1 + k) + (np.exp((k + 1)*t_end) - 1))/(k + 1), np.exp(t_end*k)])
    else:
        sol_end = np.array([np.exp(-t_end)*(1 + t_end), np.exp(-t_end)])
    if func == 1:
        def j(t, u): return (t+1)*u[0]
        def res(t):
            if k != -1: return (np.exp(-t)*(-k**3*(2 + t) + np.exp((1 + k)*t)*(-1 + k + k*t)))/(k**2*(1 + k))
            else:       return -np.exp(-t)*(5 + 4*t + t**2)
    elif func == 2:
        def j(t, u): return (t+1)*u[1]
        def res(t):
            return (np.exp(k*t)*(-1 + k + k*t))/k**2
    elif func == 10:
        def j(t, u): return (t+1)*u[0]*u[1]
        def res(t):
            if k != -1: return ((np.exp((-1 + k)*t)*(4*k**3*(-2 + k + (-1 + k)*t) 
                                + np.exp((1 + k)*t)*(-1 + k)**2*(-1 + 2*k*(1 + t))))/(4*(-1 + 
                                  k)**2*k**2*(1 + k)))
            else:       return -(1/4)*np.exp(-2*t)*(5 + 6*t + 2*t**2)
    elif func == 21:
        def j(t, u): return (t+1)*u[0]**2
        def res(t):
            if k != -1: return ((np.exp(-2*t)*(-(-1 + k)**2*k**4*(3 + 2*t) + 8*np.exp((1 + k)*t)*k**3*(-2 + k - t + k*t) + 
                                np.exp(2*(1 + k)*t)*(-1 + k)**2*(-1 + 2*k*(1 + t))))/(4*k**2*(-1 + k**2)**2))
            else:       return -(1/8)*np.exp(-2*t)*(19 + 30*t + 18*t**2 + 4*t**3)
    elif func == 32:
        def j(t, u): return (t+1)*u[1]**3
        def res(t):
            return (np.exp(3*k*t)*(-1 + 3*k*(1 + t)))/(9*k**2)
    return j, res(t_end) - res(0), j(t_end, sol_end)
        
def dt_new_EPS(dt, err, tol, p): # EPS
    return dt*(tol/err)**(1/(p+1))
    
def run(tol, func):
    j, ref_j, sol_end = get_func(func)
    t, j_sum, steps = 0, 0, 0
    u = np.copy(u0)
    dt = tol**(1/4)
    while t < t_end:
        dt = min(t_end - t, dt)
        unew, ulow, umid = scheme(t, dt, u)
        j_sum += dt/6*(j(t, u) + 4*j(t + dt/2, umid) + j(t+dt, unew)) # Simpson rule
        #j_sum += dt/2*(j(t, u) + j(t+dt, unew)) # trapezoidal rule
        t += dt
        
        dt_c = dt_new_EPS(dt, abs(j(t+dt, unew) - j(t+dt, ulow)), tol, 3)
        dt = min(max(0.01*dt, dt_c), 3*dt)
        u = unew
        steps += 1
    return abs(j_sum - ref_j), steps, abs(j(t, unew) - sol_end)
    
tol_list = np.array([10**(-i) for i in range(15)])
solns = {'tol': tol_list}
func_list = [1, 2, 10, 21, 32]
leg = ['$(t+1)u_1$','$(t+1)u_2$','$(t+1)u_1 u_2$','$(t+1)u_1^2$','$(t+1)u_2^3$']
for func in func_list:
    errs, steps, errs_end = [], [], []
    for tol in tol_list:
        err, st, err_end = run(tol, func)
        errs.append(err)
        steps.append(st)
        errs_end.append(err_end)
    solns[func] = (errs, steps, errs_end)
    

import itertools
fs = 32
markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')
    
fig, ax = pl.subplots(figsize = (12, 10))
ax.loglog(tol_list, tol_list, 'k--', lw = 2)
for func in func_list:
    ax.loglog(tol_list, solns[func][0], '-o', lw = 3, ms = 17, color = next(colors), marker = next(markers))
ax.set_xlim(left = tol_list[0], right = tol_list[-1]) # automatically inverts x axis
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.legend([r'$\tau$'] + leg, fontsize = fs - 4)
ax.set_title(r'Tolerance vs. Error', fontsize = fs + 2)
ax.set_xlabel(r'Tolerance $\tau$', fontsize = fs)
ax.set_ylabel(r'$e^J$', rotation = 0, fontsize = fs + 10, labelpad = 20)
pl.savefig('4th_order_tol_vs_err.eps', dpi = 100)

markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')

fig, ax = pl.subplots(figsize = (12, 10))
ax.loglog(tol_list**(-1/4), tol_list, '--k', lw = 2)
for func in func_list:
    ax.loglog(solns[func][1], solns[func][0], '-o', lw = 3, ms = 17, color = next(colors), marker = next(markers))
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.legend(['4th order'] + leg, fontsize = fs - 4)
ax.set_title('Steps vs. Error', fontsize = fs + 2)
ax.set_xlabel('Steps', fontsize = fs)
ax.set_ylabel(r'$e^J$', rotation = 0, fontsize = fs +10, labelpad = 20)
pl.savefig('4th_order_steps_vs_err.eps', dpi = 100)
              
markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')
              
fig, ax = pl.subplots(figsize = (12, 10))
ax.loglog(tol_list, tol_list, 'k--', lw = 2)
for func in func_list:
    ax.loglog(tol_list, solns[func][2], '-o', lw = 3, ms = 17, color = next(colors), marker = next(markers))
ax.set_xlim(left = tol_list[0], right = tol_list[-1]) # automatically inverts x axis
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.legend([r'$\tau$'] + leg, fontsize = fs - 4)
ax.set_title(r'Tolerance vs. Error at $t_e$', fontsize = fs + 2)
ax.set_xlabel(r'Tolerance $\tau$', fontsize = fs)
ax.set_ylabel(r'$j(e(t_e))$', rotation = 0, fontsize = fs, labelpad = 20)
pl.savefig('4th_order_tol_vs_err_end.eps', dpi = 100)