# -*- coding: utf-8 -*-
"""
Created on Wed Feb  7 15:42:08 2018
@author: Peter Meisrimel
"""
from __future__ import division
import scipy as sp
import pylab as pl
import numpy as np
pl.close('all')

t_end = 2
k = -1
A = np.array([[-1., 1.],[0, k]])

def f(t, x):
    return A@x

u0 = np.array([1., 1.])

def scheme(t, dt, u):
    cn = np.linalg.solve(np.eye(2) - dt/2*A, (np.eye(2) + dt/2*A)@u)
    ie = np.linalg.solve(np.eye(2) - dt*A, u)
    return cn, ie

def ex_sol(t, u):
    return sp.linalg.expm(t*A)@u
    
def get_func(func):
    if k != -1:
        sol_end = np.array([np.exp(-t_end)*((1 + k) + (np.exp((k + 1)*t_end) - 1))/(k + 1), np.exp(t_end*k)])
    else:
        sol_end = np.array([np.exp(-t_end)*(1 + t_end), np.exp(-t_end)])
    if func == 1:
        def j(u): return u[0]
        def res(t):
            if k != -1: return np.exp(-t)*(-k**2 + np.exp((1 + k)*t))/(k*(1 + k))
            else:       return -np.exp(-t)*(2 + t)
    elif func == 2:
        def j(u): return u[1]
        def res(t):
            return np.exp(k*t)/k
    elif func == 10:
        def j(u): return u[0]*u[1]
        def res(t):
            if k != -1: return (np.exp((-1 + k)*t)*(2*k**2 - np.exp((1 + k)*t) + k*(2 + (-2 + np.exp((1 + k)*t)))))/(2*k*(-1 + k**2))
            else:       return -(1/4)*np.exp(-2*t)*(3 + 2*t)
    elif func == 21:
        def j(u): return u[0]**2
        def res(t):
            if k != -1: return ((np.exp(-2*t)*(np.exp(2*(1 + k)*t)*(-1 + k) + 4*np.exp((1 + k)*t)*k**2 - 
                                (-1 + k)*k**3))/(2*(-1 + k)*k*(1 + k)**2))
            else:       return -(1/4)*np.exp(-2*t)*(2 + (1 + 2*t + 2*t**2) + 2*(1 + 2*t))
    elif func == 32:
        def j(u): return u[1]**3
        def res(t):
            return (np.exp(3*k*t))/(3*k)
    return j, res(t_end) - res(0), j(sol_end)
        
def dt_new_EPS(dt, err, tol, p): # EPS
    return dt*(tol/err)**(1/(p+1))
    
def run(tol, func):
    j, ref_j, sol_end = get_func(func)
    t, j_sum, steps = 0, 0, 0
    u = np.copy(u0)
    dt = tol**(1/2)
    print('running tol = {}'.format(tol))
    ts = []
    dts = []
    while t < t_end:
        dt = min(t_end - t, dt)
        
        ts.append(t)
        dts.append(dt)
        unew, ulow = scheme(t, dt, u)
        j_sum += dt/2*(j(u) + j(unew)) # trapezoidal rule
        t += dt
        
        dt_c = dt_new_EPS(dt, abs(j(unew) - j(ulow)), tol, 1)
        dt = dt_c
        #dt = min(max(0.01*dt, dt_c), 3*dt)
        u = unew
        steps += 1
    return abs(j_sum - ref_j), steps, abs(j(unew) - sol_end), ts, dts
    
tol_list = np.array([10**(-i) for i in range(15)])
solns = {'tol': tol_list}
func_list = [1]
leg = ['$u_1$']
for func in func_list:
    errs, steps, errs_end = [], [], []
    for tol in tol_list:
        err, st, err_end, ts, dts = run(tol, func)
        errs.append(err)
        steps.append(st)
        errs_end.append(err_end)
    solns[func] = (errs, steps, errs_end)

import itertools
fs = 32
markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')
    
fig, ax = pl.subplots(figsize = (12, 10))
ax.loglog(tol_list, tol_list, 'k--', lw = 2)
for func in func_list:
    ax.loglog(tol_list, solns[func][0], '-o', lw = 3, ms = 17, color = next(colors), marker = next(markers))
ax.set_xlim(left = tol_list[0], right = tol_list[-1]) # automatically inverts x axis
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.legend([r'$\tau$'] + leg, fontsize = fs - 4)
ax.set_title('Tolerance vs. Error', fontsize = fs + 2)
ax.set_xlabel(r'Tolerance $\tau$', fontsize = fs)
ax.set_ylabel(r'$e^J$', rotation = 0, fontsize = fs, labelpad = 20)

markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')

fig, ax = pl.subplots(figsize = (12, 10))
ax.loglog(tol_list**(-1/2), tol_list, '--k', lw = 2)
for func in func_list:
    ax.loglog(solns[func][1], solns[func][0], '-o', lw = 3, ms = 17, color = next(colors), marker = next(markers))
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.legend(['2nd order'] + leg, fontsize = fs - 4)
ax.set_title('Steps vs. Error', fontsize = fs + 2)
ax.set_xlabel('Steps', fontsize = fs)
ax.set_ylabel(r'$e^J$', rotation = 0, fontsize = fs, labelpad = 20)

for f in func_list:
    print('APPROXIMATE ORDERS func = {}, (TOL vs ERR, ERR vs STEPS):'.format(f))
    for i in range(len(tol_list) - 1):
        print(np.log10(solns[f][0][i]/solns[f][0][i+1]),
              np.log10(solns[f][0][i+1]/solns[f][0][i])/np.log10(solns[f][1][i]/solns[f][1][i+1]))
              
markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')
              
fig, ax = pl.subplots(figsize = (12, 10))
ax.loglog(tol_list, tol_list, 'k--', lw = 2)
for func in func_list:
    ax.loglog(tol_list, solns[func][2], '-o', lw = 3, ms = 17, color = next(colors), marker = next(markers))
ax.set_xlim(left = tol_list[0], right = tol_list[-1]) # automatically inverts x axis
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.legend([r'$\tau$'] + leg, fontsize = fs - 4)
ax.set_title(r'Tolerance vs. Error at $t_e$', fontsize = fs + 2)
ax.set_xlabel(r'Tolerance $\tau$', fontsize = fs)
ax.set_ylabel(r'$j(e(t_e))$', rotation = 0, fontsize = fs, labelpad = 20)

markers = itertools.cycle('o^D*s')
colors = itertools.cycle('gbrckmy')
              
fig, ax = pl.subplots(figsize = (12, 10))
ax.semilogy(ts, dts, 'k-', lw = 2)
ax.grid(b = 'on', which = 'major')
ax.tick_params(labelsize=20)
ax.set_title(r'Timesteps for $\tau = 10^{-14}$', fontsize = fs + 2)
ax.set_xlabel('t', fontsize = fs)
ax.set_ylabel(r'$\Delta t$', rotation = 0, fontsize = fs + 5, labelpad = 20)
fig.savefig('timesteps_tol_14.eps', dpi = 100)