# -*- coding: utf-8 -*-
"""
Created on Thu Dec 10 16:09:34 2015
@author: Peter Meisrimel
"""
from __future__ import division
from run_simulation import run_simulation
import numpy as np
import pylab as pl

## Comparison of new & old controller with DWR over various tolerances
# \n run_mode = 1
class run_tolerances_default(run_simulation):
    ## Basic initialization
    # @param self pointer to self
    # @param file_read parameters read from file
    # @param legends collected legend entries if run mode requires several files
    # @return None
    def __init__(self, file_read, legends):
        run_simulation.__init__(self, file_read, legends)
        self.fontsize = 32
        self.tol_list = [10**-i for i in range(file_read['tol_step_start'], file_read['tol_step_start'] + file_read['tol_steps'])]
        if file_read['tol_steps_dwr'] is None:
            file_read['tol_steps_dwr'] = file_read['tol_steps']
        self.tol_list_dwr = [10**-i for i in range(file_read['tol_step_start'], file_read['tol_step_start'] + file_read['tol_steps_dwr'])]
        self.fig_tol_err_int, self.ax_tol_err_int = pl.subplots(figsize = (12, 10))
        self.fig_tol_err_int_save = 'tols_vs_error_' + self.save_file_add
        
        self.ax_tol_err_int.set_title(r'Tolerance vs. Error', fontsize = self.fontsize + 2)
        self.ax_tol_err_int.set_xlabel(r'Tolerance $\tau$', fontsize = self.fontsize)
        self.ax_tol_err_int.set_ylabel(r'$e^J$', rotation = 0, labelpad = 20, fontsize = self.fontsize + 10)
        self.ax_tol_err_int.set_xscale('log'); self.ax_tol_err_int.set_yscale('log')
        self.ax_tol_err_int.tick_params(labelsize=20)
        self.ax_tol_err_int.grid(b = 'on', which = 'major')
        self.ax_tol_err_int.set_ylim(top = 10)
        self.ax_tol_err_int.set_xlim(left = self.tol_list[0], right = self.tol_list[-1]) # automatically inverts x axis
        self.ax_tol_err_int.plot(self.tol_list, self.tol_list, '--k', lw = 2)
        
        self.fig_tol_work, self.ax_tol_times = pl.subplots(figsize = (12, 10))
        self.fig_tol_work_save = 'tols_tol_vs_work_' + self.save_file_add
        self.ax_tol_times.set_title(r'Tolerance  vs. Work', fontsize = self.fontsize + 2)
        self.ax_tol_times.set_xlabel(r'Tolerance $\tau$', fontsize = self.fontsize)
        self.ax_tol_times.set_ylabel('time', labelpad = 20, fontsize = self.fontsize)
        self.ax_tol_times.set_xscale('log'); self.ax_tol_times.set_yscale('log')
        self.ax_tol_times.tick_params(labelsize=20)
        self.ax_tol_times.grid(b = 'on', which = 'major')
        self.ax_tol_times.set_xlim(left = self.tol_list[0], right = self.tol_list[-1]) # automatically inverts x axis
        
        self.fig_err_work, self.ax_err_work = pl.subplots(figsize = (12, 10))
        self.fig_err_work_save = 'comp_efficiency_' + self.save_file_add
        self.ax_err_work.set_title(r'Computational efficiency', fontsize = self.fontsize + 2)
        self.ax_err_work.set_xlabel(r'Comp. time', fontsize = self.fontsize)
        self.ax_err_work.set_ylabel(r'$e^J$', rotation = 0, fontsize = self.fontsize + 10, labelpad = 20)
        self.ax_err_work.set_xscale('log'); self.ax_err_work.set_yscale('log')
        self.ax_err_work.grid(b = 'on', which = 'major')
        self.ax_err_work.tick_params(labelsize=20)
        
        self.fig_err_steps, self.ax_err_steps = pl.subplots(figsize = (12, 10))
        self.fig_err_steps_save = 'tols_error_vs_steps_' + self.save_file_add
        self.ax_err_steps.set_title(r'Error in functional vs steps', fontsize = self.fontsize + 2)
        self.ax_err_steps.set_xlabel(r'steps', fontsize = self.fontsize)
        self.ax_err_steps.set_ylabel(r'$e^J$', rotation = 0, fontsize = self.fontsize + 10, labelpad = 20)
        self.ax_err_steps.set_xscale('log'); self.ax_err_steps.set_yscale('log')
        self.ax_err_steps.grid(b = 'on', which = 'major')
        self.ax_err_steps.tick_params(labelsize=20)
        
        self.fig_tol_steps, self.ax_tol_steps = pl.subplots(figsize = (12, 10))
        self.fig_tol_steps_save = 'tols_tol_vs_steps_' + self.save_file_add
        self.ax_tol_steps.set_title(r'$\tau$ vs. time steps', fontsize = self.fontsize + 2)
        self.ax_tol_steps.set_xlabel(r'Tolerance $\tau$', fontsize = self.fontsize)
        self.ax_tol_steps.set_ylabel(r'steps', rotation = 90, fontsize = self.fontsize)
        self.ax_tol_steps.set_xscale('log'); self.ax_tol_steps.set_yscale('log')
        self.ax_tol_steps.grid(b = 'on', which = 'major')
        self.ax_tol_steps.tick_params(labelsize=20)
        self.ax_tol_steps.set_xlim(left = self.tol_list[0], right = self.tol_list[-1]) # automatically inverts x axis
        
        self.fig_quality, self.ax_quality = pl.subplots(figsize = (12, 10))
        self.fig_quality_save = 'grid_quality_' + self.save_file_add
        self.ax_quality.set_title(r'Grid Quality', fontsize = self.fontsize + 2)
        self.ax_quality.set_xlabel('# grid points', fontsize = self.fontsize)
        self.ax_quality.set_ylabel(r'$e^J$', rotation = 0, fontsize = self.fontsize + 10, labelpad = 20)
        self.ax_quality.set_xscale('log'); self.ax_quality.set_yscale('log')
        self.ax_quality.grid(b = 'on', which = 'major')
        self.ax_quality.tick_params(labelsize=20)
        
    ## main function for executing a run
    # @param self pointer to self
    # @param file_read parameters read from file
    # @return None
    def simulate(self, file_read):
        if file_read['save_results']:
            import time
            file_t = time.strftime("%Y-%m-%d-%H-%M-%S")    
            self.myfile = open(file_read['saving_path'] + 'tol_' + file_read['filename_self'] + '_{}.txt'.format(file_t), 'w')            
            self.save_file_header(file_read)
            self.save_file_write(('tol_list', self.tol_list))

        # error & estimate lists
        errs_new, errs_old, errs_dwr, errs_real, grid_x, grid_y, steps_new, steps_old, steps_dwr, qual_dwr = [],[],[],[],[],[],[],[],[],[]
        # time measurement lists
        times_new, times_old, times_dwr = [],[],[0]
        
        startup_done = False
        self.mesh, est_list = None, None
        err_dwr, j_prev = np.inf, 0
        for tol in self.tol_list[:1] + self.tol_list: # for startup
            if not startup_done:
                print('...performing startup with tol = %.2e' % (tol))
            else:
                print('...running tol = %.2e' % (tol))
             
            Time_integ_new, dt0_new = self.init_parameters(file_read, tol = tol, err_est_type = 1)
            Time_integ_old, dt0_old = self.init_parameters(file_read, tol = tol, err_est_type = 0)
            
            err_new, time_new, step_new = Time_integ_new.run(dt0_new)
            err_old, time_old, step_old = Time_integ_old.run(dt0_old)
            
            if tol in self.tol_list_dwr:
                Time_integ_DWR = self.init_parameters(file_read, tol = tol, DG = 2, dwr_scheme = file_read['dwr_scheme'])
                err_dwr, time_dwr, err_real, est_list, j_prev, step_dwr = Time_integ_DWR.run(self.mesh, est = err_dwr, est_list = est_list, j_prev = j_prev)
                time_dwr = time_dwr + times_dwr[-1]
                if len(steps_dwr) == 0: # skip initial grid
                    pass
                elif step_dwr != steps_dwr[-1]: # add number of steps done to new total
                    step_dwr += steps_dwr[-1]
                self.mesh = Time_integ_DWR.mesh
            
            if startup_done:
                errs_new.append(err_new)
                times_new.append(time_new)
                steps_new.append(step_new)
                
                errs_old.append(err_old)
                times_old.append(time_old)
                steps_old.append(step_old)
                
                if tol in self.tol_list_dwr:
                    errs_dwr.append(err_dwr)
                    times_dwr.append(time_dwr)
                    errs_real.append(err_real)
                    steps_dwr.append(step_dwr)
                    qual_dwr.append(self.mesh.num_cells())
            else:
                startup_done = True
                if tol in self.tol_list_dwr:
                    self.mesh, est_list = None, None
                    err_dwr, j_prev = np.inf, 0
        # get time normalization factor t_min
        times_new = np.array(times_new)
        times_old = np.array(times_old)
        times_dwr = np.array(times_dwr[1:])
        t_min = min(min(times_new), min(times_old), min(times_dwr))
        
        self.reset_markers()      
        self.ax_tol_err_int.plot(self.tol_list, errs_new, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_err_int.plot(self.tol_list, errs_old, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_err_int.plot(self.tol_list_dwr, errs_dwr, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_err_int.plot(self.tol_list_dwr, errs_real, marker = next(self.markers), c = next(self.colors))
            
        self.reset_markers()
        self.ax_tol_times.plot(self.tol_list, times_new/t_min, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_times.plot(self.tol_list, times_old/t_min, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_times.plot(self.tol_list_dwr, times_dwr/t_min, marker = next(self.markers), c = next(self.colors))
        
        self.reset_markers()
        self.ax_err_work.plot(times_new/t_min, errs_new, marker = next(self.markers), c = next(self.colors))
        self.ax_err_work.plot(times_old/t_min, errs_old, marker = next(self.markers), c = next(self.colors))
        self.ax_err_work.plot(times_dwr/t_min, errs_dwr, marker = next(self.markers), c = next(self.colors))
        self.ax_err_work.plot(times_dwr/t_min, errs_real, marker = next(self.markers), c = next(self.colors))
        
        self.reset_markers()
        self.ax_tol_steps.plot(self.tol_list, steps_new, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_steps.plot(self.tol_list, steps_old, marker = next(self.markers), c = next(self.colors))
        self.ax_tol_steps.plot(self.tol_list_dwr, steps_dwr, marker = next(self.markers), c = next(self.colors))
        
        self.reset_markers()
        self.ax_err_steps.plot(steps_new, errs_new, marker = next(self.markers), c = next(self.colors))
        self.ax_err_steps.plot(steps_old, errs_old, marker = next(self.markers), c = next(self.colors))
        self.ax_err_steps.plot(steps_dwr, errs_dwr, marker = next(self.markers), c = next(self.colors))
        self.ax_err_steps.plot(steps_dwr, errs_real, marker = next(self.markers), c = next(self.colors))
        
        self.reset_markers()
        self.ax_quality.plot(steps_new, errs_new, marker = next(self.markers), c = next(self.colors))
        self.ax_quality.plot(steps_old, errs_old, marker = next(self.markers), c = next(self.colors))
        self.ax_quality.plot(qual_dwr, errs_dwr, marker = next(self.markers), c = next(self.colors))
        self.ax_quality.plot(qual_dwr, errs_real, marker = next(self.markers), c = next(self.colors))
        
        #for j in range(len(self.tol_list)):
        #    self.ax_err_work.plot(grid_x[j]/t_min, grid_y[j], '--k', lw = 1)
            
        if file_read['save_results']:
            self.save_file_write(('errs_new', errs_new), ('errs_old', errs_old), ('errs_dwr', errs_dwr),
                                 ('grid_x', grid_x), ('grid_y', grid_y), ('times_new', times_new),
                                 ('times_old', times_old), ('times_dwr', times_dwr), ('errs_real', errs_real),
                                 ('steps_new', steps_new), ('steps_old', steps_old), ('steps_dwr', steps_dwr),
                                 ('qual_dwr', qual_dwr))
            self.save_file_end()

    ## General function for priting plots
    # @param self pointer to self
    # @param file_read parameters read from file
    # @param dpi dpi value for image saving
    # @return None
    def print_plot(self, file_read, dpi = 100):
        lab = ['Goal Or.', 'Classic', 'DWR Est', 'DWR Err']
        self.ylim(self.ax_tol_err_int)
        self.ax_tol_err_int.legend([r'$\tau$'] + lab, loc = 0, fontsize = self.fontsize - 4)
        self.fig_tol_err_int.savefig(file_read['saving_path'] + self.fig_tol_err_int_save + self.save_format, dpi = dpi)
        
        self.ax_tol_times.legend(lab, loc = 0, fontsize = self.fontsize - 4 )
        self.fig_tol_work.savefig(file_read['saving_path'] + self.fig_tol_work_save + self.save_format, dpi = dpi)
        
        self.ax_err_work.legend(lab, loc = 0, fontsize = self.fontsize - 4)
        self.fig_err_work.savefig(file_read['saving_path'] + self.fig_err_work_save + self.save_format, dpi = dpi)
        
        self.ax_err_steps.legend(lab, loc = 0, fontsize = self.fontsize - 4)
        self.fig_err_steps.savefig(file_read['saving_path'] + self.fig_err_steps_save + self.save_format, dpi = dpi)
        
        self.ax_tol_steps.legend(lab, loc = 0, fontsize = self.fontsize - 4)
        self.fig_tol_steps.savefig(file_read['saving_path'] + self.fig_tol_steps_save + self.save_format, dpi = dpi)
        
        self.ax_quality.legend(lab, loc = 0, fontsize = self.fontsize - 4)
        self.fig_quality.savefig(file_read['saving_path'] + self.fig_quality_save + self.save_format, dpi = dpi)
        
## Comparison various input 
# \n run_mode = 2
class run_tolerances(run_tolerances_default):
    ## Basic initialization
    # @param self pointer to self
    # @param file_read parameters read from file
    # @param legends collected legend entries if run mode requires several files
    # @return None
    def __init__(self, file_read, legends):
        run_tolerances_default.__init__(self, file_read, legends)
        self.fig_tol_err_int_save = 'tols_tol_vs_error_' + self.save_file_add + '_t_' + str(file_read['problem_t_end']).replace('.', ',')
        self.fig_tol_work_save = 'tols_tol_vs_work_' + self.save_file_add + '_t_' + str(file_read['problem_t_end']).replace('.', ',')
        self.fig_err_work_save = 'tols_error_vs_work_' + self.save_file_add + '_t_' + str(file_read['problem_t_end']).replace('.', ',')
        self.fig_err_steps_save = 'tols_error_vs_steps_' + self.save_file_add + '_t_' + str(file_read['problem_t_end']).replace('.', ',')
        self.ax_err_steps.set_title(r'Grid Quality', fontsize = self.fontsize + 2)
        self.ax_err_steps.set_xlabel('# grid points', fontsize = self.fontsize)
        self.fig_tol_steps_save = 'tols_tol_vs_steps_' + self.save_file_add + '_t_' + str(file_read['problem_t_end']).replace('.', ',')
        self.times, self.errs, self.steps, self.times_red, self.errs_red, self.legs, self.legs_red = [],[],[],[],[],[],[]
        self.t_min = np.inf
        if file_read['save_results']:
            import time
            file_t = time.strftime("%Y-%m-%d-%H-%M-%S")    
            self.myfile = open(file_read['saving_path'] + 'tol_' + file_read['filename_self'] + '_{}.txt'.format(file_t), 'w')            
            self.save_file_header(file_read)
            self.save_file_write(('tol_list', self.tol_list))
        
        self.tol_fig_ax = []
        
        for tol in self.tol_list[:1] + self.tol_list: # first entry twice for startup
            fig, ax = pl.subplots(figsize = (12, 10))
            fig_save = 'tols_' + self.save_file_add + '_t_' + str(file_read['problem_t_end']) + '_tol_' + '%0.0e' % (tol)
            ax.set_title(r'Timestepsizes, $\tau = 10^{%2i}$' % (np.log10(tol)), fontsize = self.fontsize + 2)
            ax.set_xlabel(r't', fontsize = self.fontsize)
            ax.set_ylabel(r'$\Delta t$', labelpad = 20, fontsize = self.fontsize, rotation = 0)
            ax.set_yscale('log')
            ax.set_xlim([0, file_read['problem_t_end']])
            ax.grid(b = 'on', which = 'major')
            ax.tick_params(labelsize=20)
            self.tol_fig_ax.append((tol, fig, ax, fig_save))
        
    ## main function for executing a run
    # @param self pointer to self
    # @param file_read parameters read from file
    # @return None
    def simulate(self, file_read):
        # error & estimate lists
        self.DWR = file_read['dwr_scheme'] # > 0 for DWR case
        errs, steps, times, errs_real = [],[],[],[]
        if self.DWR: # DWR case
            times.append(0) # as times are adding up previous comp. times
            self.mesh, est_list = None, None
            err, j_prev = np.inf, 0
        
        startup_done = False
        m = next(self.markers); col = next(self.colors); ls = next(self.linestyles)
        for (tol, fig, ax, fig_save) in self.tol_fig_ax:
            if not startup_done:
                print('...performing startup with tol = %.2e' % (tol))
            else:
                print('...running tol = %.2e' % (tol))
            
            # initialize and run
            if self.DWR:
                Time_integ = self.init_parameters(file_read, tol = tol, DG = 2, dwr_scheme = file_read['dwr_scheme'])
                err, time_run, err_real, est_list, j_prev, step = Time_integ.run(self.mesh, est = err, est_list = est_list, j_prev = j_prev)
                time_run = time_run + times[-1]
                self.mesh = Time_integ.mesh
            else:
                Time_integ, dt0 = self.init_parameters(file_read, tol = tol, diagnostics = 1)
                err, time_run, step = Time_integ.run(dt0)
                stats = Time_integ.stats
            
            # saving of results
            if startup_done:
                errs.append(err)
                times.append(time_run)
                steps.append(step)
                if self.DWR:
                    errs_real.append(err_real)
                else:
                    if len(stats['t'][:-1]) > 80: m = None
                    ax.plot(stats['t'][:-1], np.diff(np.array(stats['t'])), marker = m, c = col, linestyle = ls)
            else:
                startup_done = True
                if self.DWR:
                    self.mesh, est_list = None, None
                    err, j_prev = np.inf, 0
        
        if self.DWR: times = times[1:] # remove zero again
        self.times.append(np.array(times))
        self.errs.append(np.array(errs))
        self.times_red.append(np.array(times))
        self.errs_red.append(np.array(errs))
        self.steps.append(np.array(steps))
        if self.DWR:
            self.times.append(np.array(times))
            self.errs.append(np.array(errs_real))
            self.legs.append(file_read['legend'] + ' Est')
            self.legs_red.append(file_read['legend'] + ' est')
            self.legs.append(file_read['legend'] + ' Err')
            self.steps.append(np.array(steps))
        else:
            self.legs.append(file_read['legend'])
            self.legs_red.append(file_read['legend'])
            
        self.t_min = min(self.t_min, min(times))
        
        if file_read['save_results']:
            self.myfile.write('\n' + file_read['filename_self'])
            self.save_file_write(('errs', errs), ('times', times), ('steps', steps))
            if self.DWR:
                self.save_file_write(('errs_real', errs_real))

    ## General function for priting plots
    # @param self pointer to self
    # @param file_read parameters read from file
    # @param dpi dpi value for image saving
    # @return None
    def print_plot(self, file_read, dpi = 100):
        if file_read['save_results']: self.save_file_end()
        self.reset_markers()
        markers = [next(self.markers) for i in range(len(self.errs))]
        colors = [next(self.colors) for i in range(len(self.errs))]
        
        for i in range(len(self.errs)):
            self.ax_tol_err_int.plot(self.tol_list, self.errs[i], marker = markers[i], c = colors[i])
        self.reset_markers()
        for i in range(len(self.errs)):
            self.ax_tol_times.plot(self.tol_list, self.times[i]/self.t_min, marker = markers[i], c = colors[i])
            self.ax_err_work.plot(self.times[i]/self.t_min, self.errs[i], marker = markers[i], c = colors[i])
            self.ax_err_steps.plot(self.steps[i], self.errs[i], marker = markers[i], c = colors[i])
            self.ax_tol_steps.plot(self.tol_list, self.steps[i], marker = markers[i], c = colors[i])
                              
        self.ylim(self.ax_tol_err_int)
        self.ax_tol_err_int.legend([r'$\tau$'] + self.legs, loc = 0, fontsize = self.fontsize - 4)
        self.fig_tol_err_int.savefig(file_read['saving_path'] + self.fig_tol_err_int_save + self.save_format, dpi = dpi)
        
        self.ax_tol_times.legend(self.legs_red, loc = 0, fontsize = self.fontsize - 4 )
        self.fig_tol_work.savefig(file_read['saving_path'] + self.fig_tol_work_save + self.save_format, dpi = dpi)
        
        self.ax_err_work.legend(self.legs_red, loc = 0, fontsize = self.fontsize - 4)
        self.fig_err_work.savefig(file_read['saving_path'] + self.fig_err_work_save + self.save_format, dpi = dpi)
        
        self.ax_err_steps.legend(self.legs_red, loc = 0, fontsize = self.fontsize - 4)
        self.fig_err_steps.savefig(file_read['saving_path'] + self.fig_err_steps_save + self.save_format, dpi = dpi)
        
        self.ax_tol_steps.legend(self.legs_red, loc = 0, fontsize = self.fontsize - 4)
        self.fig_tol_steps.savefig(file_read['saving_path'] + self.fig_tol_steps_save + self.save_format, dpi = dpi)
        
        for (tol, fig, ax, fig_save) in self.tol_fig_ax[1:]:
            ax.legend(self.legs_red, loc = 0, fontsize = self.fontsize - 4)
            fig.savefig(file_read['saving_path'] + fig_save + self.save_format, dpi = dpi)