# -*- coding: utf-8 -*-
"""
Created on Fri Aug 12 13:38:07 2016
@author: Peter Meisrimel
"""
from __future__ import division
import pylab as pl
import numpy as np
from run_simulation import run_simulation

## Basic order verification of DG form schemes via error in endpoint
# \n run_mode = 10
class run_DG_verify(run_simulation):
    ## Basic initialization
    # @param self pointer to self
    # @param file_read parameters read from file
    # @param legends collected legend entries if run mode requires several files
    # @return None
    def __init__(self, file_read, legends):
        run_simulation.__init__(self, file_read, legends)
        self.fontsize = 32
        
        self.fig, self.ax = pl.subplots(figsize = (12, 10))
        self.fig_save = 'convergence_verify_' + self.save_file_add
        
        self.ax.set_title(r'convergence order verification', fontsize = self.fontsize + 2)
        self.ax.set_xlabel(r'$\Delta t$', fontsize = self.fontsize)
        self.ax.set_ylabel('y', labelpad = 20, rotation = 90, fontsize = self.fontsize)
        self.ax.set_xscale('log'); self.ax.set_yscale('log')
        self.ax.grid(b = 'on', which = 'major')
        self.ax.tick_params(labelsize=20)
        
        self.dt_list = [(file_read['problem_t_end'])/(2.**i) for i in range(1, file_read['tol_steps'] + 1)]
        self.ax.set_xlim(left = self.dt_list[0], right = self.dt_list[-1])
        
    ## main function for executing a run
    # @param self pointer to self
    # @param file_read parameters read from file
    # @return None
    def simulate(self, file_read):
        err_end_list = []

        for dt in self.dt_list: # for startup
            Time_integ, _ = self.init_parameters(file_read, tol = 0)
            
            # Run with basic dt controller + DWR estimation
            y_final = Time_integ.run(dt0 = dt, adaptive = 0, final = True)
            
            ex = Time_integ.get_sol_point()
            try: ## DG
                sol = y_final.vector().array()
            except: ## RK
                sol = y_final
                pl.plot(y_final, c = next(self.colors))
            err_end_list.append(np.linalg.norm(ex - sol))
        self.ax.plot(self.dt_list, err_end_list, marker = next(self.markers), c = 'k')
        print('Order estimation through the slopes, first component : ')
        for i in range(len(err_end_list) - 1):
            slope = pl.log(err_end_list[i]/err_end_list[i+1])/pl.log(2)
            print(slope)
        
    ## General function for priting plots
    # @param self pointer to self
    # @param file_read parameters read from file
    # @param dpi dpi value for image saving
    # @return None
    def print_plot(self, file_read, dpi = 100):
        self.ax.legend([r'$u_1$', r'$u_2$'], loc = 0, fontsize = self.fontsize - 4)
        self.fig.savefig(file_read['saving_path'] + self.fig_save + self.save_format, dpi = dpi)